import { IoIosArrowForward, IoIosArrowBack } from 'react-icons/io';
import React, { useState, useEffect } from "react"
import { IconButton, Stack } from "@mui/material"

import ItemSlot from "./Slot"
import DragAndDropAPI from "./DragAndDropAPI"

const inventoryLimit = 9

const InventoryContent = ({
  nftImage,
  onMouseUp,
  onMouseDown,
  goNextPage,
  goPrevPage,
}) => {
  const [state, setState] = useState({
    url: "",
    metadata: "",
    selectedid: "",
    userid: "",
    anchorEl: null,
    selectedtype: "",
    inventory: [],
    currentPage: 1,
  })

  const totalPage = Math.ceil(nftImage?.length / inventoryLimit)

  const [items, setItems] = useState([...nftImage])
  const [draggingSlotId, setDraggingSlot] = useState(null)
  const getItemDataInSlot = (slot) => items.find((item) => item.slot === slot)

  const swapItemSlots = (oldSlot, newSlot) => {
    setItems((currentState) => {
      let newInventory = [...currentState]
      let oldIndex: any, newIndex: any

      // Finding the old ones..

      newInventory.forEach((item, index) => {
        if (item.slot === oldSlot) {
          oldIndex = index
        } else if (item.slot === newSlot) {
          newIndex = index
        }
      })

      // Replacing them

      newInventory[oldIndex] = { ...newInventory[oldIndex], slot: newSlot }
      newInventory[newIndex] = { ...newInventory[newIndex], slot: oldSlot }

      return [...newInventory]
    })
  }

  const moveItemToSlot = (oldSlot, newSlot) => {
    setItems((currentState) => {
      let newInventory = [...currentState]
      let targetIndex: any
      let isChange: any = true
      let selectSlot: any
      newInventory.forEach((item, index) => {
        if (item.slot === oldSlot) {
          targetIndex = index
          selectSlot = item
        }
        if (item.slot === parseInt(newSlot)) {
          isChange = false
        }
      })
      if (isChange) {
        newInventory[targetIndex] = {
          ...newInventory[targetIndex],
          slot: parseInt(newSlot),
        }
      }
      onMouseDown(selectSlot, newSlot)
      return [...newInventory]
    })
  }

  const onInventoryItemDragged = ({ detail: eventData }: any) => {
    const oldSlot = parseInt(eventData.slot),
      newSlot = parseInt(eventData.destination.slot)

    if (eventData.destination.type === "empty-slot") {
      moveItemToSlot(oldSlot, newSlot)
    } else if (eventData.destination.type === "item") {
      swapItemSlots(oldSlot, newSlot)
    }
  }

  // ***********************************

  const goToNextPage = () => {
    setState((prevState) => ({
      ...prevState,
      currentPage: prevState.currentPage + 1,
    }))
    goNextPage()
  }
  const goToPrevPage = () => {
    setState((prevState) => ({
      ...prevState,
      currentPage: prevState.currentPage > 1 ? prevState.currentPage - 1 : 1,
    }))
    goPrevPage()
  }
  const getCurrentSlots = () => {
    const res: any = []
    const startIndex = (state.currentPage - 1) * inventoryLimit
    const endIndex = state.currentPage * inventoryLimit
    for (let i = startIndex; i < endIndex; i++) res.push(i)
    return res
  }

  useEffect(() => {
    const selectItme = items.find((item) => item.slot === draggingSlotId)
    if (selectItme) onMouseUp(selectItme, draggingSlotId)
  }, [draggingSlotId, items])

  useEffect(() => {
    console.log(state);
  }, [state.currentPage])

  return (
    <div className="inventory-panel">
      <div className="inventory-text">Inventory</div>
      {items.length !== 0 ? (
        <div className="inventory-items">
          {/* drag & drop API integration */}
          <DragAndDropAPI
            activeDraggedSlot={draggingSlotId}
            setActiveDraggedSlot={setDraggingSlot}
            changeItems={moveItemToSlot}
          />

          {/* inventory grid */}
          <Stack
            direction="row"
            justifyContent="center"
            flexWrap={"wrap"}
            sx={{ position: "relative" }}
            className={`inventory`}
          >
            {getCurrentSlots().map((slot) => {
              return (
                <ItemSlot
                  slot={slot}
                  value={getItemDataInSlot(slot) || null}
                  key={slot}
                />
              )
            })}
          </Stack>
          {/* pagination */}
          <Stack
            direction="row"
            justifyContent="space-between"
            alignItems="center"
          >
            <IconButton
              sx={{ svg: { color: "white" } }}
              className={`${"inventory-paginationBtn"} ${
                state.currentPage <= 1 ? "inventory-disable" : ""
              }`}
              onClick={() => goToPrevPage()}
              disabled={state.currentPage < 1 ? true : false}
            >
              <IoIosArrowBack />
            </IconButton>
            <p className="inventory-text">
              Page {`${state.currentPage} - ${totalPage}`}
            </p>

            <IconButton
              sx={{ svg: { color: "white" } }}
              className={`${"inventory-paginationBtn"} ${
                state.currentPage >= totalPage ? "inventory-disable" : ""
              }`}
              onClick={() => goToNextPage()}
              disabled={state.currentPage >= totalPage ? true : false}
            >
              <IoIosArrowForward />
            </IconButton>
          </Stack>
        </div>
      ) : (
        <Stack>
          <p className="inventory-text">No Data Found</p>
        </Stack>
      )}
    </div>
  )
}

export default InventoryContent
